/**
 * Copyright (c) 2012, Martin Foster <martin@ethereal-realms.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Ethereal-Realms.org nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lyra.Block;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.lyra.AbstractBlock;
import org.lyra.Ledger;
import org.lyra.Widgets.InfiniteProgressPanel;


/**
 * The org.lyra.Widgets.InfiniteProgressPanel class is the preferred method
 * for blocking the user interface.  However it can lead to slow downs due to
 * the graphics intensive method.
 * <p>
 * @author Martin Foster
 * @version 1.0.5, %Revision, 174%, %LastChangedDate, 27/10/12 3:33PM%
 * @see InfiniteProgressPanel
 */
public class InfiniteProgressImpl extends AbstractBlock {

	/**
	 * Identifier for this class allowing us to set it apart from other
	 * implementations which currently exist.
	 */
	public static final String IDENT = "pane";
	
	// Name of class
	private static final String CLASS = "org.lyra.Block.InfiniteProgressImpl";
	
	// Class Objects
	private InfiniteProgressPanel Block;
	
	// Initial Message
	private String Message;
	
	// Ledger Instance
	private Ledger Logger;
	
	
	/**
	 * Constructor for the class.  Creates an instance of the 
	 * <i>InfiniteProgressPanel</i> will interface an application class with 
	 * a proper interface.
	 * <p>
	 * @param logger Instance of ledger for logging errors and warnings.
	 * @param msg Sting message containing initial statement to display.
	 */
	public InfiniteProgressImpl(Ledger logger, String msg) {
		
		// Pass Up
		this.Logger  = logger;
		this.Message = msg;
	}
	// End of Constructor Method
	
	
	/**
	 * Provides an instance of the InfiniteProgressPanel that can be
	 * implemented into another instance.
	 * <p>
	 * @return Component capable of being dropped into a JGlassPane. 
	 */
	public synchronized InfiniteProgressPanel getPane() {
		
		// Check for Existence
		if (Block == null) {
			
			// Create new base instance
			Block = new InfiniteProgressPanel(Message);
		}
		
		// Return Instance
		return Block;
		
	}
	// End of getPane Method
	
	
	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#startBlock()
	 */
	public void startBlock() {
		
		/*
		 * Now that we know we will be updating the document it is time
		 * to start blocking the interface.  We will also lock step with 
		 * the worker thread to ensure it takes place in the time it needs.
		 */
		try {
			SwingUtilities.invokeAndWait(new Runnable() {

				/* (non-Javadoc)
				 * @see java.lang.Runnable#run()
				 */
				public void run() {
					
					// Trigger sync in worker thread
					getPane().start();
				}
			});
		} 
		
		/*
		 *  Unlike invokeLater this method may throw an exception log them 
		 *  accordingly.
		 */
		catch (InterruptedException e1) {
			Logger.Warn(CLASS, "syncLibrary", e1);
		} 
		
		catch (InvocationTargetException e1) {
			Logger.Warn(CLASS, "syncLibrary", e1);
		}
		
	}
	// End of startBlock method

	
	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#stopBlock()
	 */
	public void stopBlock() {
		
		/*
		 * Now that we know we will be updating the document it is time
		 * to start blocking the interface.  We will also lock step with 
		 * the worker thread to ensure it takes place in the time it needs.
		 */
		try {
			SwingUtilities.invokeAndWait(new Runnable() {

				/* (non-Javadoc)
				 * @see java.lang.Runnable#run()
				 */
				public void run() {
					
					// Trigger sync in worker thread
					getPane().stop();
				}
			});
		} 
		
		/*
		 *  Unlike invokeLater this method may throw an exception log them 
		 *  accordingly.
		 */
		catch (InterruptedException e1) {
			Logger.Warn(CLASS, "syncLibrary", e1);
		} 
		
		catch (InvocationTargetException e1) {
			Logger.Warn(CLASS, "syncLibrary", e1);
		}
	}
	// End of stopBlock method


	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#setMessage(java.lang.String)
	 */
	public void setMessage(String msg) {

		// Direct pass-through
		getPane().setText(msg);
	}
	// End of setMessage Method

	
	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#getMessage()
	 */
	public String getMessage() {

		// Direct pass-through
		return getPane().getText();
	}
	// End of getMessage Method
}
