/**
 * Copyright (c) 2012, Martin Foster <martin@ethereal-realms.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Ethereal-Realms.org nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lyra.Interface.Chart;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.commons.lang3.text.WordUtils;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.MultiplePiePlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.CategoryToPieDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.util.Rotation;
import org.jfree.util.TableOrder;

import org.lyra.Ledger;
import org.lyra.Lyra;
import org.lyra.Database.Comptroller;
import org.lyra.Interface.Constants;
import org.lyra.Interface.Chart.SaveDialog;
import org.lyra.Listener.WindowSizeListener;
import org.lyra.Widgets.RapidDialog;
import org.lyra.Widgets.ResizeIcon;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;



/**
 * This is an additional tool which allows to view certain pieces of
 * information captured from the Audit table in chart form.  Top artists,
 * albums, genres and so forth.
 * <p>
 * @author Martin Foster
 * @version 1.0.5, %Revision, 174%, %LastChangedDate, 27/10/12 3:33PM%
 * @since 1.0.5
 */
public class DisplayPanel extends JFrame {

	// Static requirements
	private static final long serialVersionUID = 1L;
	
	/*
	 * VISUAL ELEMENTS
	 */
	
	// Control Bar
	private JPanel panelControl = null;
	
	// Statistics Source Elements
	private JLabel labelStat = null;
	private JComboBox comboStat = null;
	
	// Chart Options
	private JLabel labelChart = null;
	private JComboBox comboChart = null;

	// Entries limiter
	private JLabel labelNumber = null;
	private JComboBox comboNumber = null;
	
	// Time period to measure
	private JLabel labelPeriod = null;
	private JComboBox comboPeriod = null;
	private JToggleButton toggleGroup = null;
	
	// Legend Feature
	private JToggleButton toggleLegend = null;
	
	// Control Button
	private JButton buttonGenerate = null;
	private JButton buttonSave = null;

	// Viewing Port
	private JScrollPane scrollViewer = null;
	
	// Window Resizing
	private JPanel panelStatus = null;
	private JLabel iconResize = null;
	

	/*
	 * PACKAGE ELEMENTS
	 */
	
	// Database Instance
	private Comptroller Manager = null;
	
	// Logger Instance
	private Ledger Logger = null;
	
	// Language Element
	private Map<String,String> Lang;
	
	// Global Settings
	private Map<String,String> Global;	
	
	
	/*
	 * CLASS ELEMENTS
	 */
	
	// Populated elements for lookup
	private Map<String,String> MCharts;
	private Map<String,String> MStats;
	private Map<String,String> MPeriods;
	
	// Display elements
	private String[] Charts;
	private String[] Stats;
	private String[] Periods;
	private String[] Numbers;

	// Interaction capability
	private Interact Interface = new Interact();
	
	JFrame Parent = null;
	
	
	/*
	 * CONSTANTS
	 */
	 
	// Screen minimum
	private static final int MIN_WIDTH = 600;
	private static final int MIN_HEIGHT = 440;
	 
	// Minimum and Maximum selection
	private static final int MIN_SELECT = 5;
	private static final int MAX_SELECT = 25;
	
	// Periods
	private static final int MAX_PERIODS = 6;
	

	/**
	 * Primary constructor for the class, which will establish interactive
	 * and functional elements.
	 * <p>
	 * @param logger Ledger instance for logging of actions.
	 * @param manager Instance of Database Comptroller.
	 * @param global Global settings map.
	 * @param lang Language independent component.
	 */
	@SuppressWarnings("serial")
	public DisplayPanel(Ledger logger, Comptroller manager, Map<String,String> global, final Map<String,String> lang) {
		
		// Have super class initialise
		super();
		
		// Pass up as necessary
		Global = global;
		Parent = this;
		
		/*
		 * When we start generating the interface, we need to actually populate
		 * drop down menus with necessary information.
		 */
		
		// Statistical Source
		MStats = new HashMap<String,String>(){
		  {
			put(lang.get("Graphs.Stat.Fav.Album"), "Album");
			put(lang.get("Graphs.Stat.Fav.Artist"), "Artist");
			put(lang.get("Graphs.Stat.Fav.Genre"), "Genre");
			put(lang.get("Graphs.Stat.Fav.Song"), "Songs");
			put(lang.get("Graphs.Stat.Rated"), "Ratings");
		  }
		};
		Stats = new String[] {
			lang.get("Graphs.Stat.Fav.Album"),
			lang.get("Graphs.Stat.Fav.Artist"),
			lang.get("Graphs.Stat.Fav.Genre"),
			lang.get("Graphs.Stat.Fav.Song"),
			lang.get("Graphs.Stat.Rated")
		};
		
		// Group Search Capability
		MCharts = new HashMap<String,String>(){
		  {
			put(lang.get("Graphs.Type.Bar"),"Bar");
			put(lang.get("Graphs.Type.Bar3D"),"Bar3D");
			put(lang.get("Graphs.Type.Pie"),"Pie");
			put(lang.get("Graphs.Type.Pie3D"),"Pie3D");
		  }
		};
		Charts = new String[] {
			lang.get("Graphs.Type.Bar"),
			lang.get("Graphs.Type.Bar3D"),
			lang.get("Graphs.Type.Pie"),
			lang.get("Graphs.Type.Pie3D")
		};
		
		// Periodic Search Range
		MPeriods = new HashMap<String,String>(){
		  {
			put(lang.get("Graphs.Period.Global"),"100 years");
			put(lang.get("Graphs.Period.Week.One"),"7 days");
			put(lang.get("Graphs.Period.Week.Two"),"14 days");
			put(lang.get("Graphs.Period.Month.One"),"1 months");
			put(lang.get("Graphs.Period.Month.Three"),"3 months");
			put(lang.get("Graphs.Period.Month.Six"),"6 months");
			put(lang.get("Graphs.Period.Year"),"1 years");
		  }
		};
		Periods = new String[] {
			lang.get("Graphs.Period.Global"),
			lang.get("Graphs.Period.Week.One"),
			lang.get("Graphs.Period.Week.Two"),
			lang.get("Graphs.Period.Month.One"),
			lang.get("Graphs.Period.Month.Three"),
			lang.get("Graphs.Period.Month.Six"),
			lang.get("Graphs.Period.Year")
		};	
		
		// Pass Up
		this.Manager = manager;
		this.Logger  = logger;
		this.Lang    = lang;
		
		// Establish the look and feel
		initFrame();
	}
	// End of Constructor Method
	

	/**
	 * This method initialises this frame.   Contains setup necessary to
	 * fully establish look and feel for both the Visual Editor and application
	 * constructors.
	 */
	private void initFrame() {
		
		// Language Elements
		this.setTitle(
			Lang.get("Graphs.Window.Title"));
		
		// Adjust Icon
		this.setIconImage(new ImageIcon(
			getClass().getResource(
				Lyra.IMG_SOURCE + Lyra.LYRA)).getImage()
			);
		
		// Establish identifier
		this.setName("STATS");
		
		// Default Close
		this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		this.addWindowListener(Interface);
		
		/*
		 * Generating the numbers is fairly simple.   They will not be generate
		 * through other means.  Since this component is general to the two
		 * constructors, it has been placed here.
		 */
		Numbers = new String[MAX_SELECT - MIN_SELECT + 1];
		
		// Loop through and populate
		for (int count=MIN_SELECT; count<=MAX_SELECT; count++)
			Numbers[count-MIN_SELECT] = Integer.toString(count);
		
		
		// Set the layout
		this.setLayout(new BorderLayout());
		
		
		// Insert elements
		this.add(getPanelControl(), BorderLayout.NORTH);
		this.add(getScrollViewer(), BorderLayout.CENTER);
		this.add(getPanelStatus(), BorderLayout.SOUTH);
		
		/*
		 * In many circumstance we opt to pack the displayed elements and
		 * create an optimal screen size.  However this tends to fail unless
		 * we create a default chart.  So here we set an absolute minimun
		 * that will work for all displays and that can be adjusted from
		 * there.
		 */
		this.setMinimumSize(
			new Dimension(MIN_WIDTH, MIN_HEIGHT));
			
		/*
		 * The user has a fair amount of say on how this window will look
		 * like.   So we bring in their choices at this point for position
		 * and dimension handling.
		 */
		 
		 // Extract point
		 Point pos = new Point(
			Integer.parseInt(Global.get("State.Graph.Window.XCoord")),
			Integer.parseInt(Global.get("State.Graph.Window.YCoord"))
		  );
		
		// Position window
		this.setLocation(pos);
		
		// Physical size of the window
		this.setSize(
			Integer.parseInt(Global.get("State.Graph.Window.Width")),
			Integer.parseInt(Global.get("State.Graph.Window.Height"))
		  ); 
		
		// Physical state of the window
		this.setState(
			Integer.parseInt(Global.get("State.Graph.Window.State")));
	}
	// End of initFrame Method
	

	/**
	 * This method initialises labelChart	
	 * 	
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelChart() {
		
		// Check for existence
		if (labelChart == null) {
			
			// Create instance
			labelChart = new JLabel();
			
			// Apply image
			labelChart.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Graph"))));
			
			// Language Elements
			labelChart.setToolTipText(
				Lang.get("Graphs.Tooltip.Chart"));
		}
		
		// Return Instance
		return labelChart;
	}
	// End of getLabelChart
	
	
	/**
	 * This method initialises labelStat	
	 * 	
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelStat() {
		
		// Check for existence
		if (labelStat == null) {
			
			// Create instance
			labelStat = new JLabel();
			
			// Apply image
			labelStat.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Stat"))));
			
			// Language Elements
			labelStat.setToolTipText(
				Lang.get("Graphs.Tooltip.Stat"));
		}
		
		// Return Instance
		return labelStat;

	}
	// End of getLabelStat
	
	
	/**
	 * This method initialises labelNumber	
	 * 	
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelNumber() {
		
		// Check for existence
		if (labelNumber == null) {
			
			// Create instance
			labelNumber = new JLabel();
			
			// Apply image
			labelNumber.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Number"))));
			
			// Language Elements
			labelNumber.setToolTipText(
				Lang.get("Graphs.Tooltip.Number"));
		}
		
		// Return Instance
		return labelNumber;

	}
	// End of getLabelNumber
	
	
	/**
	 * This method initialises labelPeriod	
	 * 	
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelPeriod() {
		
		// Check for existence
		if (labelPeriod == null) {
			
			// Create instance
			labelPeriod = new JLabel();
			
			// Apply image
			labelPeriod.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Period"))));
			
			// Language Elements
			labelPeriod.setToolTipText(
				Lang.get("Graphs.Tooltip.Period"));
		}
		
		// Return Instance
		return labelPeriod;

	}
	// End of getLabelPeriod
	
	
	/**
	 * This method initialises comboStat	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getComboStat() {
		
		// Check for Existence
		if (comboStat == null) {
			
			// Create Instance
			comboStat = new JComboBox(Stats);
			
			// Language Elements
			comboStat.setToolTipText(
				Lang.get("Graphs.Tooltip.Stat"));
			
			// Identifier and Listener
			comboStat.setName("STAT");
			
			// Retrieve proper default
			comboStat.setSelectedItem(
				getDefault(
					MStats,
					Global.get("State.Graph.Control.Stat"), 
					Stats[0]
				 )
			 );
		}
		
		// Return Instance
		return comboStat;
	}
	// End of getComboStat
	
	
	/**
	 * This method initialises comboPeriod	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getComboPeriod() {
		
		// Check for Existence
		if (comboPeriod == null) {
			
			// Create Instance
			comboPeriod = new JComboBox(Periods);
			
			// Language Elements
			comboPeriod.setToolTipText(
				Lang.get("Graphs.Tooltip.Period"));
			
			// Identifier and Listener
			comboPeriod.setName("PERIOD");
			
			// Retrieve proper default
			comboPeriod.setSelectedItem(
				getDefault(
					MPeriods,
					Global.get("State.Graph.Control.Period"), 
					Periods[0]
				 )
			 );
		}
		
		// Return Instance
		return comboPeriod;
	}
	// End of getComboPeriod
	

	/**
	 * This method initialises comboChart	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getComboChart() {
		
		// Check for Existence
		if (comboChart == null) {
			
			// Create Instance
			comboChart = new JComboBox(Charts);
			
			// Language Elements
			comboChart.setToolTipText(
				Lang.get("Graphs.Tooltip.Chart"));
			
			// Identifier and Listener
			comboChart.setName("GRAPH");
			
			// Retrieve proper default
			comboChart.setSelectedItem(
				getDefault(
					MCharts,
					Global.get("State.Graph.Control.Chart"), 
					Charts[0]
				 )
			 );
		}
		
		// Return Instance
		return comboChart;
	}
	// End of comboChart Method

	
	/**
	 * This method initialises panelControl	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPanelControl() {
		
		// Check for Existence
		if (panelControl == null) {
			
			// Create Instance
			panelControl = new JPanel();
			
			// Establish Layout
			panelControl.setLayout(
				new BoxLayout(
					getPanelControl(),
					BoxLayout.X_AXIS
				 )
			 );

			// Insert Elements
			// Statistical Source
			panelControl.add(getLabelStat(), null);
			panelControl.add(getComboStat(), null);
			panelControl.add(Box.createHorizontalStrut(
				Constants.getSectionGap()), null);
			
			// Number of Entries to Display
			panelControl.add(getLabelNumber(), null);
			panelControl.add(getComboNumber(), null);
			panelControl.add(Box.createHorizontalStrut(
				Constants.getSectionGap()), null);
			
			// Type of Chart
			panelControl.add(getLabelChart(), null);
			panelControl.add(getComboChart(), null);
			panelControl.add(Box.createHorizontalStrut(
				Constants.getSectionGap()), null);
			
			// Period of Time
			panelControl.add(getLabelPeriod(), null);
			panelControl.add(getComboPeriod(), null);
			panelControl.add(Box.createHorizontalStrut(
				Constants.getSectionGap()), null);			
			
			// Group Toggling
			panelControl.add(getToggleGroup(), null);
			panelControl.add(getToggleLegend(), null);
			panelControl.add(Box.createHorizontalStrut(
				Constants.getSectionGap()), null);
			
			// Button Generation and Spacing
			panelControl.add(Box.createHorizontalGlue(), null);
			panelControl.add(getButtonSave(), null);
			panelControl.add(getButtonGenerate(), null);
		}
		
		// Return Instance
		return panelControl;
	}
	// End of getPanelControl Method
	

	/**
	 * This method initialises buttonGenerate	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getButtonGenerate() {
		
		// Check of Existence
		if (buttonGenerate == null) {
			
			// Create Instance
			buttonGenerate = new JButton();
			
			// Visual Elements
			buttonGenerate.setIcon(new ImageIcon(
				getClass().getResource(Lyra.getIconByMap("Stat-Generate"))));
			buttonGenerate.setToolTipText(
				Lang.get("Graphs.Tooltip.Generate"));

			// Identifier and Listener
			buttonGenerate.setName("GENERATE");
			buttonGenerate.addActionListener(Interface);
		}
		
		// Return Instance
		return buttonGenerate;
	}
	// End of getButtonGenerate Method
	
	
	/**
	 * This method initialises buttonSave	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getButtonSave() {
		
		// Check of Existence
		if (buttonSave == null) {
			
			// Create Instance
			buttonSave = new JButton();
			
			// Visual Elements
			buttonSave.setIcon(new ImageIcon(
				getClass().getResource(Lyra.getIconByMap("Stat-Save"))));
			buttonSave.setToolTipText(
				Lang.get("Graphs.Tooltip.Save"));

			// Identifier and Listener
			buttonSave.setName("SAVE");
			buttonSave.addActionListener(Interface);
			
			// By default we disable
			buttonSave.setEnabled(false);
		}
		
		// Return Instance
		return buttonSave;
	}
	// End of getButtonSave Method
	
	
	/**
	 * This method initialises toggleGroup	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getToggleGroup() {
		
		// Check of Existence
		if (toggleGroup == null) {
			
			// Create Instance
			toggleGroup = new JToggleButton();
			
			// Language Element
			toggleGroup.setToolTipText(
				Lang.get("Graphs.Tooltip.Group"));

			// Icon Handling
			toggleGroup.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Group-False"))));
			toggleGroup.setRolloverIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Group-False"))));
			toggleGroup.setSelectedIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Group-True"))));
						
			// Identifier and Listener
			toggleGroup.setName("GROUP");
			
			// Default is what the user wishes
			toggleGroup.setSelected(
				Boolean.parseBoolean(
					Global.get("State.Graph.Control.Group")
				 )
			 );
		}
		
		// Return Instance
		return toggleGroup;
	}
	// End of getButtonGroup Method
	
	/**
	 * This method initialises toggleLegend	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getToggleLegend() {
		
		// Check of Existence
		if (toggleLegend == null) {
			
			// Create Instance
			toggleLegend = new JToggleButton();
			
			// Language Element
			toggleLegend.setToolTipText(
				Lang.get("Graphs.Tooltip.Legend"));

			// Icon Handling
			toggleLegend.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Legend-False"))));
			toggleLegend.setRolloverIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Legend-False"))));
			toggleLegend.setSelectedIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Stat-Legend-True"))));
						
			// Identifier and Listener
			toggleLegend.setName("LEGEND");
			
			// Default is what the user wishes
			toggleLegend.setSelected(
				Boolean.parseBoolean(
					Global.get("State.Graph.Control.Legend")
				 )
			 );
		}
		
		// Return Instance
		return toggleLegend;
	}
	// End of getButtonLegend Method
	

	/**
	 * This method initialises comboNumber	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getComboNumber() {
		
		// Check for Existence
		if (comboNumber == null) {

			// Create Instance
			comboNumber = new JComboBox(Numbers);

			// Language Elements
			comboNumber.setToolTipText(
				Lang.get("Graphs.Tooltip.Number"));

			// Identifier and Listener
			comboNumber.setName("NUMBER");
			
			// We extract default from the database
			comboNumber.setSelectedItem(
				Global.get("State.Graph.Control.Number"));
		}
		
		// Return Instance
		return comboNumber;
	}
	// End of getFieldNumber Method

	
	/**
	 * This method initialises scrollViewer	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getScrollViewer() {
		
		// Check for Existence
		if (scrollViewer == null) {
			
			// Create Instance and Establish Viewport
			scrollViewer = new JScrollPane();
		}
		
		// Return Instance
		return scrollViewer;
	}
	// End of getScrollViewer Method
	

	/**
	 * This method initialises panelWindow.  Contains nothing more then the
	 * resize element for creating a consistent look and feel.	
	 * <p>
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPanelStatus() {
		
		// Check for Existence
		if (panelStatus == null) {
			
			// Create Instance
			panelStatus = new JPanel();
			panelStatus.setLayout(
				new BoxLayout(
					getPanelStatus(), 
					BoxLayout.X_AXIS
				 )
			 );

			// Add in resizing label/icon
			panelStatus.add(Box.createHorizontalGlue());
			panelStatus.add(getIconResize(), null);
		}
		
		// Return Instance
		return panelStatus;
	}
	// End of getPanelStatus Method
	
	
	/**
	 * This method initialises iconResize.  
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getIconResize() {
		
		// Verify existence of element
		if (iconResize == null) {

			// Create and establish baseline
			iconResize = new JLabel(new ResizeIcon());
			iconResize.setOpaque(false);
			
			/*
			 * Making this icon do something of use is one of the core reasons
			 * why it was added to the display itself.  Here we create an 
			 * instance and add the listener, since it is not used anywhere else
			 * we don't worry out creating an instance earlier.
			 */
			WindowSizeListener sizer = new WindowSizeListener(this);
			
			// Add in listener elements
			iconResize.addMouseListener(sizer);
			iconResize.addMouseMotionListener(sizer);
		}
		
		// Return label
		return iconResize;
	}
	// End of getIconResize Method

	
	/**
	 * This method will recover the default value based on what is stored in 
	 * the database.  There is no direct way to extract the information from
	 * the HashMap itself, so this will do it for us.
	 * <p>
	 * @param source Source map containing value conversion.
	 * @param compare String to compare.
	 * @param base Default value assigned normally.
	 * @return String containing default value.
	 */
	private String getDefault(Map<String, String> source, String compare, String base) {
	
		// Default value
		String selected = base;
		
		// Loop through keys and determine if its a match
		for (String current: source.keySet()) {
	
			// When a match occurs we just return the value
			if (source.get(current).equals(compare)) {
				
				selected = current;
				break;
			}
		}
		
		// Return default
		return selected;
	}
	// End of getDefault Method
	
	
	/**
	 * The code separation from the user interface.   This Class handles all
	 * of the complexities of making the interface interactive.
	 * <p>
	 * @author Martin Foster
	 */
	private class Interact implements ActionListener, WindowListener {

		// Class Identifier
		private static final String CLASS = "org.lyra.Interface.ChartsAndStatsPanel.Interact";

		// Chart element
		ChartPanel myChart;
		
	
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent evt) {
			
			// We only handle buttons
			if (!(evt.getSource() instanceof JButton))
				return;
			
			
			// Extract source information
			String source = ((JButton)evt.getSource()).getName();
			
			
			/*
			 * The following section deals exclusively with the most complex
			 * button/function.   This will generate the actual material and
			 * display it on the panel.
			 */
			if (source.equals(getButtonGenerate().getName())) {

				// Log the event
				Logger.Info(
					CLASS,
					"actionPerformed",
					Lang.get("Graphs.Log.Generate")
					    .replaceAll("MTYPE",    (String)getComboChart().getSelectedItem())
					    .replaceAll("MSTAT",    (String)getComboStat().getSelectedItem())
					    .replaceAll("MPERIOD",  (String)getComboPeriod().getSelectedItem())
					    .replaceAll("MRESULTS", (String)getComboNumber().getSelectedItem())
				  );
				
				
				/*
				 * We need to translate the contents of the combo boxes and 
				 * use them to generate values accordingly.  Fortunately 
				 * they are already in a Map<String,String> element to speed
				 * things along.
				 */
				
				// Period to search from
				String period = MPeriods.get(
					(String)getComboPeriod().getSelectedItem());
				
				// Type of Graph
				String chart = MCharts.get(
					(String)getComboChart().getSelectedItem());
				
				// Statistical source
				String stat = MStats.get(
					(String)getComboStat().getSelectedItem());

				
				/*
				 * Now we have to capture the state for future times.  So we
				 * take our generated values and pass them on as necessary.
				 */
				 
				// Standard elements require no translation
				Global.put("State.Graph.Control.Period", period);
				Global.put("State.Graph.Control.Chart", chart);
				Global.put("State.Graph.Control.Stat", stat);
				Global.put("State.Graph.Control.Number", (String)getComboNumber().getSelectedItem());
				 
				// Group and Legend are somewhat unique
				Global.put("State.Graph.Control.Group", 
				 	Boolean.toString(getToggleGroup().isSelected()));
				Global.put("State.Graph.Control.Legend", 
					Boolean.toString(getToggleLegend().isSelected())); 
				
				
				/*
				 * Log the attempt to generate the diagram generation here. 
				 * This will also allow us to see all of the options selected
				 * prior to actual generation.
				 */
				Logger.Info(
					CLASS, 
					"actionPerformed", 
					Lang.get("Graphs.Generate.Log")
						.replaceAll("MPERIOD", Global.get("State.Graph.Control.Period"))
						.replaceAll("MCHART",  Global.get("State.Graph.Control.Chart"))
						.replaceAll("MSTAT",   Global.get("State.Graph.Control.Stat"))
						.replaceAll("MNUMBER", Global.get("State.Graph.Control.Number"))
						.replaceAll("MGROUP",  Global.get("State.Graph.Control.Group"))
						.replaceAll("MLEGEND", Global.get("State.Graph.Control.Legend"))
				  );
				
				
				/*
				 * The title bar is one of those components that has to
				 * change based on the presence of groups.   Groups affect
				 * us accordingly.
				 */
				String title = (getToggleGroup().isSelected())
					? Lang.get("Graphs.Title.Grouped")
					 	 .replaceAll("MTYPE",   (String)getComboChart().getSelectedItem())
						 .replaceAll("MSTAT",   (String)getComboStat().getSelectedItem())
						 .replaceAll("MPERIOD", (String)getComboPeriod().getSelectedItem())
						 .replaceAll("MITERATION", Integer.toString(MAX_PERIODS))
					: Lang.get("Graphs.Title.Combined")
						 .replaceAll("MTYPE",   (String)getComboChart().getSelectedItem())
						 .replaceAll("MSTAT",   (String)getComboStat().getSelectedItem())
						 .replaceAll("MPERIOD", (String)getComboPeriod().getSelectedItem());
				
				// Title capitalisation
				title = WordUtils.capitalizeFully(title);
				 
				 
				 
								
				/*
				 * We need to extract the data set for which the rest of the
				 * generation will take place upon.  Since we can opt to
				 * analyze the information over a period of time it is
				 * necessary to build the following elements in a very basic
				 * and modular fashion.
				 */
				 
				// Data set
				Map<String, String> dset;
				
				
				// How many times will we be looping
				int mperiod = getToggleGroup().isSelected() 
					? MAX_PERIODS 
					: 1;
				
				// This provides the start and end period for the element
				int speriod = Manager.getStatTimescale("0 days");
				int eperiod = Manager.getStatTimescale(period);
				
				// Original method for calculation
				int operiod = eperiod;
				
				// Default holder for all charts
				DefaultCategoryDataset data = new DefaultCategoryDataset();
				
				// We loop through the necessary amount of times to build up
				for (int count=0; count < mperiod; count++) {
					
					// These are special circumstances
					if ((stat.equals("Songs"))
					 || (stat.equals("Ratings"))) {
					
						// Are we sorting based on songs
						Boolean rating = (stat.equals("Ratings"))
							? true
							: false;
				
						// Extract the data set in its entirety
						dset = Manager.getStatDatasetSong(
							Integer.toString(speriod),
							Integer.toString(eperiod),
							(String)getComboNumber().getSelectedItem(),
							rating,
							Lang.get("Graphs.Lang.Joiner")
						  );
					}
				
					// Default handling for the rest
					else {
					
						// Extract the data set in its entirety
						dset = Manager.getStatDatasetGeneral(
							stat,
							Integer.toString(speriod),
							Integer.toString(eperiod),
							(String)getComboNumber().getSelectedItem()
						  );
					}
				
				
					/*
					 * We need to add in a bit of language to split the
					 * groups of queries as necessary.  So we look at the
					 * time elapsed and work accordingly.
					 */
					 
					// Format numbers
					NumberFormat nf = NumberFormat.getInstance();
					nf.setMinimumFractionDigits(1);

					// Holder
					String section;

					 
					// Over Days
					if (period.equals("7 days")) {
					
					 	// Format String
					 	section = Lang.get("Graphs.Bar.Period.Day")
					 	  			  .replaceAll("MCOUNT", Integer.toString(eperiod));
					 }
					
					 // Week scale
					 else if (period.equals("14 days")) {
							
						// Format String
						section = Lang.get("Graphs.Bar.Period.Week")
							 	  	  .replaceAll("MCOUNT", Integer.toString(eperiod/7));
					}					  

					// Month scale
					else if ((period.equals("1 months")) 
					 || (period.equals("3 months"))
					 || (period.equals("6 months"))) {
							
						// Format String
						section = Lang.get("Graphs.Bar.Period.Month")
							 	  	   .replaceAll("MCOUNT", Integer.toString(eperiod/30));
					}	
					 
					 
					// Years
					else {
					 	 
						// Format String
						section = Lang.get("Graphs.Bar.Period.Year")
							 	  	   .replaceAll("MCOUNT", Integer.toString(eperiod));
					 }
					 
					 
					/*
					 * We only need to generate the dataset once.  Building 
					 * a Category dataset will in fact allow us to convert 
					 * to a pie one when required.
					 */
					if ((getToggleGroup().isSelected())
					 && (chart.startsWith("Bar"))){
					 	 
					 	/*
					 	 * Bar charts look off when looking at it from
					 	 * a grouped period of time.  So we inverse
					 	 * information in order to make it look correct
					 	 * though this is the only time you will see it.
					 	 */
					 	  
					 	// Loop and populate
					 	for (String key: dset.keySet())
					 	 	data.setValue(
					 	 		Integer.parseInt(dset.get(key)),
					 	 		key, 
					 	 		section
					 	 	 );
					}
					 
					else {
					 	 
					 	/*
					 	 * All other fields use the same formula.  So we
					 	 * default to this method.
					 	 */
					 	 
					 	// Loop and populate
					 	for (String key: dset.keySet())
					 	 	data.setValue(
					 	 		Integer.parseInt(dset.get(key)),
					 	 		section, 
					 	 		key
					 	 	 );
					 }
					 

					/*
					 * Now that we have generated the necessary data we
					 * we ready for another potential pass.
					 */
					speriod =  eperiod;
					eperiod += operiod;
				}
			
				// Holder Chart
				JFreeChart fchart = null;
					
				/*
				 * There are two major types of diagrams, so we need to
				 * discover which we are using in order to create the
				 * initial data set.
				 */
				 
				// Pie Charts
				if (chart.startsWith("Pie")) {
					
					/*
					 * Multiple pie charts use different generators to get
					 * going.  So this is where we make the selection.
					 */
					if (getToggleGroup().isSelected()) {
					
						/*
						 * Now we need to know if we are going to be going 3D or
						 * instead keep a 2D look and feel.
						 */
						 if (chart.endsWith("3D"))	{
						
						 	 // Generate the chart
						 	fchart = ChartFactory.createMultiplePieChart3D(
						 		title,							// Title section
						 		data,							// Generated Database
						 		TableOrder.BY_ROW,				// How to arrange rows
						 		getToggleLegend().isSelected(),	// Generate Legend
						 		true,							// Support Tooltips
						 		false							// Generate URLS
						 	   );
					  
						 	//  Get the base element
						 	MultiplePiePlot plot = (MultiplePiePlot)fchart.getPlot();
						 	JFreeChart subchart = plot.getPieChart();
						 	PiePlot splot = (PiePlot) subchart.getPlot();
						
						 	// Change look and feel
						 	splot.setStartAngle(290);
						 	splot.setDirection(Rotation.CLOCKWISE);
						 	splot.setForegroundAlpha(0.95f);
						 } 

						 // Generic look and feel
						 else {
						
						 	// Generate the chart
						 	fchart = ChartFactory.createMultiplePieChart(
						 		title,							// Title section
						 		data,							// Generated Database
						 		TableOrder.BY_ROW,				// How to arrange rows
						 		getToggleLegend().isSelected(),	// Generate Legend
						 		true,							// Support Tooltips
						 		false							// Generate URLS
						 	  );
						  
							 //  Get the base element
					         MultiplePiePlot plot = (MultiplePiePlot)fchart.getPlot();
					         JFreeChart subchart = plot.getPieChart();
					         PiePlot splot = (PiePlot) subchart.getPlot();

					         // Change look and feel
					         splot.setStartAngle(290);
					         splot.setDirection(Rotation.CLOCKWISE);
						 }	
						
					}
						
					// Our run of the mill single diagrams
					else {
						
						// Establishes a data set for Pie Charts
						CategoryToPieDataset pdata = 
							new CategoryToPieDataset(
								data,
								TableOrder.BY_ROW,
								0
							  );
					
						/*
						 * Now we need to know if we are going to be going 3D or
						 * instead keep a 2D look and feel.
						 */
						 if (chart.endsWith("3D"))	{
						
						 	// Generate the chart
						 	fchart = ChartFactory.createPieChart3D(
						 	 	title,							// Title section
						 	 	pdata,							// Generated Database
						 	 	getToggleLegend().isSelected(),	// Generate Legend
						 	 	false,							// Support Tooltips
						 	 	Locale.getDefault()				// Locale Support
						 	   );
					  
						 	//  Change the look and feel
						 	PiePlot3D plot = (PiePlot3D)fchart.getPlot();
						
						 	plot.setStartAngle(290);
						 	plot.setDirection(Rotation.CLOCKWISE);
						 	plot.setForegroundAlpha(0.95f);
						 }

						 // Generic look and feel
						 else {
						
							// Generate the chart
							fchart = ChartFactory.createPieChart(
						 		title,							// Title section
						 		pdata,							// Generated Database
						 		getToggleLegend().isSelected(),	// Generate Legend
						 		false,							// Support Tooltips
						 		Locale.getDefault()				// Locale Support
						 	  );
						  
							//  Change the look and feel
							PiePlot plot = (PiePlot)fchart.getPlot();
						
						 	plot.setStartAngle(290);
						 	plot.setDirection(Rotation.CLOCKWISE);
						 }
					}
				}
				
				// Bar Charts
				else {

					/*
					 * Now we need to know if we are going to be going 3D or
					 * instead keep a 2D look and feel.
					 */
					if (chart.endsWith("3D"))	{
						
						// Generate the chart
						fchart = ChartFactory.createBarChart3D(
								title,										// Title section
								(String)getComboStat().getSelectedItem(),	// Category Axis
								Lang.get("Graphs.Bar.Instance"),			// Value Axis
								data,										// Generated Database
								PlotOrientation.HORIZONTAL,					// Orientation
								getToggleGroup().isSelected(),				// Generate Legend
								false,										// Support Tooltips
								false										// Generate URLS
						  );
					}
					
					// Plain two-dimensional space
					else {
						
						// Generate the chart
						fchart = ChartFactory.createBarChart(
								title,										// Title section
								(String)getComboStat().getSelectedItem(),	// Category Axis
								Lang.get("Graphs.Bar.Instance"),			// Value Axis
								data,										// Generated Database
								PlotOrientation.HORIZONTAL,					// Orientation
								getToggleGroup().isSelected(),				// Generate Legend
								false,										// Support Tooltips
								false										// Generate URLS
						  );
					}
				}
				
				// Drop in newly generated chart
				ChartPanel endproduct = new ChartPanel(fchart);
				
				// Enable certain convenience functions
				endproduct.setMouseWheelEnabled(true);
				endproduct.setMouseZoomable(true);
				
				// Drop in and replace
				getScrollViewer().setViewportView(endproduct);
				getScrollViewer().revalidate();
				
				// Reference for later use
				myChart = endproduct;
				
				/*
				 * Now we can make the choice to enable the save button
				 * since there is information there to actually generate.
				 */
				getButtonSave().setEnabled(true);
			}
			
			/*
			 * The option to save requires some information prior to 
			 * completing the evolution.   Hence we make sure to employ ask
			 * the user what is required prior to saving the file.
			 */
			else if (source.equals(getButtonSave().getName())) {
				
				// Popup Window and get results
				Map<String, String> save = SaveDialog.getVerdict(
						Lang, 
						myChart.getWidth(), 
						myChart.getHeight(), 
						Parent
					  );
				
				// No need to process if its empty
				if ((save != null) 
				 && (!save.isEmpty())) {
					
					// Capture elements
					int height = Integer.parseInt(
								save.get("Height"));
					int width  = Integer.parseInt(
								save.get("Width"));
				
					// Assign sane values if necessary
					if (height < MIN_HEIGHT)
						height = MIN_HEIGHT;
					
					if (width < MIN_WIDTH)
						width = MIN_WIDTH;
					
					/*
					 * The next elements are straight forward as it
					 * requires us going through the different types and
					 * calling in the appropriate function to deal with
					 * the transition.
					 */
					 
					// These might fault on us
					try {
					 	 // Joint Picture Experts Group
					 	 if (save.get("Type").equals("JPEG"))
					 	 	ChartUtilities.saveChartAsJPEG(
					 	 		new File(save.get("File")),
					 	 		myChart.getChart(),
					 	 		width,
					 	 		height
					 	 	  );
						
					 	 // Portable Network Graphics
					 	 else if (save.get("Type").equals("PNG"))
					 		ChartUtilities.saveChartAsPNG(
					 			new File(save.get("File")),
					 			myChart.getChart(),
					 			width,
					 			height
					 		  );
						
					 	 // Scalable Vector Graphics
					 	 else if (save.get("Type").equals("SVG"))
					 	 	saveChartAsSVG(
					 	 		new File(save.get("File")),
					 	 		myChart.getChart(),
					 	 		width,
					 	 		height
					 	 	  );
					 	 	  
					 	 // Log success
					 	 Logger.Info(
					 		CLASS, 
					 		"actionPerformed", 
					 		Lang.get("Graphs.Save.Log.Success")
					 			.replaceAll("MFILE",   save.get("File"))
					 			.replaceAll("MWIDTH",  Integer.toString(width))
					 			.replaceAll("MHEIGHT", Integer.toString(height))
					 	  );
					 }
					 
					 // Log and notify user of epic failure
					 catch (IOException e) {
					 	 
						 // Notify the user
						 RapidDialog rd = new RapidDialog(Lang);
						 
						 // Display window
						 rd.noticeFail(
							Parent,
							Lang.get("Graphs.Save.Fail")
				 				.replaceAll("MFILE",  save.get("File"))
				 				.replaceAll("MERROR", e.getLocalizedMessage())
				 		  );
						 
					 	// Log success
					 	Logger.Info(
					 		CLASS, 
					 		"actionPerformed", 
					 		Lang.get("Graphs.Save.Log.Fail")
					 			.replaceAll("MFILE",  save.get("File"))
					 			.replaceAll("MERROR", e.getLocalizedMessage())
					 	  );
					 }
				}
			}
		}
		// End of actionPerformed Method
		
		
		/*
		 * WINDOW LISTENERS
		 */

		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
		 */
		public void windowClosing(WindowEvent e) {
			
			/*
			 * There is no need to process if we are dealing with some other
			 * entity which is sending its information to us.
			 */
			if (e.getSource() instanceof JFrame) {
				
				/*
				 * We have all the elements necessary now to capture the
				 * window state prior to close off the application.  This
				 * will effectively deal with it.
				 */
				
				// Extract source
				JFrame source = (JFrame)e.getSource();
				
				/*
				 * We can only process window information if we are not minimised
				 * or maximised.  Otherwise restoring dimensions will simply affect
				 * matters negatively.
				 */
				if (source.getState() == JFrame.NORMAL) {
				
					// Window Width
					Global.put("State.Graph.Window.Width", 
						Integer.toString(source.getWidth()));
				
					// Window Height
					Global.put("State.Graph.Winodw.Height", 
						Integer.toString(source.getHeight()));
				
					// Window X-Coordinate
					Global.put("State.Graph.Window.XCoord", 
						Integer.toString((int)source.getLocationOnScreen().getX()));
				
					// Window X-Coordinate
					Global.put("State.Graph.Window.YCoord", 
						Integer.toString((int)source.getLocationOnScreen().getY()));
				}
				
			
				/*
				 * We like to return the window to its previous state.  However we
				 * do not wish to bring it back minimised, so we restore only to a
				 * state that is safe.
				 */
				if (source.getState() != JFrame.ICONIFIED) {
				
					// We do not care about returning to an icon
					Global.put("State.Graph.Window.State", 
						 Integer.toString(source.getState()));
				}
					
				/*
				 * Now that we have gotten all the states we need to recover the
				 * look and feel we now move to actually saving this information for
				 * future use.
				 */
				Manager.updateSettings(Global, "State.Graph.");
			}
		}
		// End of windowClosing Method
		
		
		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
		 */
		public void windowClosed(WindowEvent e) { /* This space left intentionally blank */ }
		
		/*
		 * (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
		 */
		public void windowActivated(WindowEvent e) { /* This space left intentionally blank */ }

		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
		 */
		public void windowDeactivated(WindowEvent e) { /* This space left intentionally blank */ }

		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
		 */
		public void windowDeiconified(WindowEvent e) { /* This space left intentionally blank */ }

		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
		 */
		public void windowIconified(WindowEvent evt) { /* This space left intentionally blank */ }
		// End of windowIconified

		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
		 */
		public void windowOpened(WindowEvent e) { /* This space left intentionally blank */ }

		
		/**
		 * Exports a JFreeChart to a SVG file.
		 * <p>
		 * @param svgFile the output file.
		 * @param chart JFreeChart to export.
		 * @param width Integer containing width dimension.
		 * @param height Integer containing height dimension.
		 * @throws IOException if writing the svgFile fails.
		 */
		private void saveChartAsSVG(File svgFile, JFreeChart chart, int width, int height) throws IOException {


			// Get a DOMImplementation and create an XML document
			DOMImplementation domImpl =
				GenericDOMImplementation.getDOMImplementation();
			Document document = domImpl.createDocument(null, "svg", null);

			// Create an instance of the SVG Generator
			SVGGraphics2D svgGenerator = new SVGGraphics2D(document);

			// Draw the chart in the SVG generator
			Rectangle bounds = new Rectangle(width, height);
			chart.draw(svgGenerator, bounds);

			// Write SVG file
			OutputStream outputStream = new FileOutputStream(svgFile);
			Writer out = new OutputStreamWriter(outputStream, "UTF-8");
			svgGenerator.stream(out, true /* use css */);						
			outputStream.flush();
			outputStream.close();
		}
		// End of exportChartAsSVG Method
		
		
	}
	// End of Interact Class
}
